<?php
/**
 * FuzeWorks CLIComponent
 *
 * The FuzeWorks PHP FrameWork
 *
 * Copyright (C) 2013-2021 i15
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * @author    i15
 * @copyright Copyright (c) 2013 - 2021, i15. (https://i15.nl)
 * @license   https://opensource.org/licenses/MIT MIT License
 *
 * @since Version 1.2.0
 *
 * @version Version 1.2.0
 */

namespace FuzeWorks\CLI;
use FuzeWorks\CLI\Exception\CLIException;
use FuzeWorks\Configurator;
use FuzeWorks\Controllers;
use FuzeWorks\Events;
use FuzeWorks\Exception\FactoryException;
use FuzeWorks\Factory;
use FuzeWorks\iComponent;
use FuzeWorks\Logger;
use FuzeWorks\Models;
use FuzeWorks\MVCRComponent;
use FuzeWorks\Priority;
use FuzeWorks\Router;
use FuzeWorks\Views;

class CLIComponent implements iComponent
{

    /**
     * Whether CLIComponent is configured to handle a CLI request
     *
     * @var bool
     */
    public static bool $willHandleRequest = false;

    public function getName(): string
    {
        return 'CLIComponent';
    }

    public function getClasses(): array
    {
        return [
            'cli' => $this,
            'cliInput' => '\FuzeWorks\CLI\Input',
            'cliOutput' => '\FuzeWorks\CLI\Output'
        ];
    }

    public function onAddComponent(Configurator $configurator)
    {
        // Add dependencies
        $configurator->addComponent(new MVCRComponent());

        // Add fallback configuration directory
        $configurator->addDirectory(
            dirname(__DIR__) . DIRECTORY_SEPARATOR . 'Config',
            'config',
            Priority::LOWEST
        );

        // If the component will handle a request, set the logger to output for CLI
        if (self::$willHandleRequest)
            $configurator->call('logger', 'setLoggerTemplate', null, 'logger_cli');

    }

    public function onCreateContainer(Factory $container)
    {
    }

    public function enableComponent()
    {
        self::$willHandleRequest = true;
    }

    public function disableComponent()
    {
        self::$willHandleRequest = false;
    }

    protected int $logCount;

    /**
     * @return bool
     * @throws CLIException
     * @throws FactoryException
     */
    public function routeCliRequest(): bool
    {
        if (!self::$willHandleRequest)
            throw new CLIException("Could not route CLI request. CLIComponent is not configured to handle requests.");

        // Load dependencies
        /** @var Router $router */
        /** @var Input $input */
        /** @var Models $models */
        /** @var Views $views */
        /** @var Controllers $controllers */
        $input = Factory::getInstance('cliInput');
        $models = Factory::getInstance('models');
        $views = Factory::getInstance('views');
        $controllers = Factory::getInstance('controllers');

        // Add the directories for Models, Views, Controllers from this Component
        $srcPath = dirname(__DIR__, 2);
        $models->addComponentPath($srcPath . DS . 'Models'. Priority::LOW);
        $views->addComponentPath($srcPath . DS . 'Views', Priority::LOW);
        $controllers->addComponentPath($srcPath . DS . 'Controllers', Priority::LOW);

        // Load the commandProcessor
        $processor = new CommandProcessor();
        Events::addListener([$processor, 'routerCallViewEventListener'], 'routerCallViewEvent', Priority::HIGHEST);

        // Start logging the request
        Logger::newLevel("Routing CLI request...");

        // Fetch input options
        $arguments = $input->arguments();

        // Prepare commandString and execute
        array_shift($arguments);
        $commandString = implode(' ', $arguments);
        $processor->executeCommand($commandString);

        // And stop logging the request
        Logger::stopLevel();

        return true;
    }
}